"""Module to hold constants for testing."""
import os
from esa import SAW

# Handle pathing.
THIS_DIR = os.path.dirname(os.path.abspath(__file__))
CASE_DIR = os.path.join(THIS_DIR, 'cases')
DUMMY_CASE = os.path.join(CASE_DIR, 'dummy_case.pwb')
DATA_DIR = os.path.join(THIS_DIR, 'data')
SNIPPET_DIR = os.path.join(THIS_DIR, '..', 'docs', 'rst', 'snippets')
SNIPPET_FILES = [os.path.join(SNIPPET_DIR, x) for x in
                 os.listdir(SNIPPET_DIR) if x.endswith('.rst')]

# Use the dummy case to get the version of Simulator.
saw = SAW(DUMMY_CASE)
VERSION = saw.version
saw.exit()
del saw

# Path to IEEE 14 bus model.
PATH_14 = os.path.join(CASE_DIR, 'ieee_14',
                       'IEEE 14 bus_pws_version_{}.pwb'.format(VERSION))
# TODO: Update all PWD files with versions when we create them.
PATH_14_PWD = os.path.join(CASE_DIR, 'ieee_14', 'IEEE 14 bus.pwd')

# Path to the Texas 2000 bus model.
PATH_2000 = os.path.join(CASE_DIR, 'tx2000',
                         'tx2000_base_pws_version_{}.pwb'.format(VERSION))

# Path to the WSCC model.
PATH_9 = os.path.join(CASE_DIR, 'wscc_9',
                      'WSCC 9 bus_pws_version_{}.pwb'.format(VERSION))

# Aux file for filtering buses by area.
AREA_AUX_FILE = os.path.join(THIS_DIR, 'area_filter.aux')

# Map cases for doc testing.
CASE_MAP = {'14': PATH_14, '2000': PATH_2000}

# Path to file containing lines for one of the examples.
CANDIDATE_LINES = os.path.join(DATA_DIR, 'CandidateLines.csv')



"""Run this with any version of Python >= 3.5 to test ESA on all
installed versions of Python. Ensure you run this from the 'tests'
directory.

Prior to running this script, you should install all supported Python
versions to your system (>= 3.5).

For help, simply run this script like:
python _test_python_versions.py -h
"""
import argparse
import os
import subprocess
import shutil

# Current directory and top-level repository directory.
THIS_DIR = os.getcwd()
TOP_DIR = os.path.abspath(os.path.join(THIS_DIR, '..'))


def main(python_install_dir, local, fresh):
    # Get Python directories.
    # https://stackoverflow.com/a/973492/11052174
    dirs = [os.path.join(python_install_dir, o) for o in
            os.listdir(python_install_dir)
            if os.path.isdir(os.path.join(python_install_dir, o))]

    # Initialize listing of output files.
    out_files = []

    # Loop.
    for d in dirs:
        # Get last two characters of d.
        code = d[-2:]
        assert code in ('35', '36', '37', '38'), 'Only Python >= 3.5!'

        print('*' * 80)
        print('Doing work for Python {}'.format(code))

        # Name of virtual environment to create.
        venv_name = os.path.join(THIS_DIR, 'test-venv-{}'.format(code))

        # Create a virtual environment if necessary.
        if os.path.isdir(venv_name) and (not fresh):
            print('No need to create {}'.format(venv_name))
        else:
            print('Creating {}...'.format(venv_name))
            if fresh:
                print('Removing existing virtual environment since the '
                      '"fresh" argument was supplied.')

                shutil.rmtree(venv_name, ignore_errors=True)

            subprocess.run((os.path.join(d, 'python.exe'),
                            '-m', 'venv', venv_name))
            print('Done.')

        # Get full path to executable Python file.
        exe = os.path.join(venv_name, 'Scripts', 'python.exe')

        # Check things are working:
        # subprocess.run((exe, '-c', 'import sys; print(sys.version);'))

        # Upgrade pip and setuptools.
        subprocess.run((exe, '-m', 'pip', 'install', '--upgrade',
                        '--no-cache-dir', 'pip', 'setuptools'))

        # Define command for installing ESA.
        cmd_tuple = \
            (exe, '-m', 'pip', 'install', '--upgrade', '--no-cache-dir')

        # Add --force-reinstall if desired.
        if fresh:
            cmd_tuple += ('--force-reinstall',)

        # Install ESA, including the test dependencies.
        if not local:
            subprocess.run(cmd_tuple + ('esa[test]',))
        else:
            subprocess.run(cmd_tuple + ('.[test]',), cwd=TOP_DIR)

        # Define output file for testing results.
        out_file = os.path.join(THIS_DIR, 'test_results_{}'.format(code))
        out_files.append(out_file)

        # Run the tests.
        print('Running tests...')
        with open(out_file, 'w') as f:
            subprocess.run(
                (exe, '-m', 'unittest', 'discover', 'tests'), cwd=TOP_DIR,
                stderr=subprocess.STDOUT, stdout=f)

        print('Done!')

    print('*' * 80)
    print('ALL DONE!')
    print('View results in the following files:')
    print(out_files)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument(
        'python_install_dir',
        help='Full path to directory where all Python installations exist.',
        type=str)
    parser.add_argument(
        '--local', action='store_true', help='Set this flag to install ESA '
        'locally from source. If this flag is not set, ESA will be installed '
        'from PyPi.')
    parser.add_argument(
        '--fresh', action='store_true', help='Set this flag to force fresh '
        'virtual environments and packages. Existing virtual environments '
        'from a previous run of this script will be removed, and packages '
        'will be installed with the --force-upgrade flag.'
    )
    args_in = parser.parse_args()
    main(python_install_dir=args_in.python_install_dir, local=args_in.local,
         fresh=args_in.fresh)




import setuptools
import os
import re

with open('VERSION', 'r') as fh:
    __version__ = fh.read()

with open("README.rst", "r") as fh:
    long_description = fh.read()

# Update the __version__ in __init__ before installing:
# Path to __init__.py:
init_path = os.path.join('esa', '__init__.py')
# Read __init__.py:
with open(init_path, 'r') as fh:
    __init__ = fh.read()

# Update the version:
__init__ = re.sub(r'__version__\s*=\s*[\'"][0-9]+\.[0-9]+\.[0-9]+[\'"]',
                  '__version__ = "{}"'.format(__version__),
                  __init__)

# Write new __init__.py:
with open(init_path, 'w') as fh:
    fh.write(__init__)

setuptools.setup(
    name='esa',
    version=__version__,
    description='Easy SimAuto (ESA): An easy-to-use Python connector to '
                'PowerWorld Simulator Automation Server (SimAuto).',
    long_description=long_description,
    long_description_content_type="text/x-rst",
    author='Zeyu Mao, Brandon Thayer, Yijing Liu',
    author_email='zeyumao2@tamu.edu, blthayer@tamu.edu, yiji21@tamu.edu',
    url='https://github.com/mzy2240/ESA',
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows",
        "Development Status :: 5 - Production/Stable",
        "Environment :: Win32 (MS Windows)",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "Natural Language :: English",
        "Topic :: Education",
        "Topic :: Scientific/Engineering",
        "Topic :: Software Development",

    ],
    keywords=['Python', 'PowerWorld', 'PowerWorld Simulator', 'Simulator',
              'PowerWorld Simulation Automation Server', 'SimAuto',
              'Automation', 'Power Systems', 'Electric Power', 'Power',
              'Easy SimAuto', 'ESA', 'Smart Grid', 'Numpy', 'Pandas'],
    install_requires=['pandas >= 0.25', 'numpy >= 1.13.3', 'scipy', 'pywin32',
                      'pypiwin32', 'networkx'],
    python_requires='>=3.5',
    # There are a couple tests that use networkx, and we use the magic
    # of sphinx for documentation. Coverage is necessary to keep the
    # coverage report up to date.
    extras_require={'test': ['networkx', 'coverage', 'matplotlib'],
                    'doc': ['sphinx', 'tabulate']},
    license='MIT',
    # TODO: Why aren't we zip safe?
    zip_safe=False
)
